@@ -42,17 +42,21 @@
 #include <limits.h>
 #include <errno.h>
 #include <assert.h>
-#include <syslog.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
 
 #ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
 #endif
 
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 #ifdef HAVE_LINUX_VM_SOCKETS_H
 #include <linux/vm_sockets.h>
+#elif HAVE_SYS_VSOCK_H
+#include <sys/vsock.h>
 #endif
 
 #include <pthread.h>
@@ -60,11 +64,15 @@
 #include <dlfcn.h>
 
 #include "ascii-string.h"
+#include "exit-with-parent.h"
+#include "nbd-protocol.h"
+#include "realpath.h"
+#include "strndup.h"
+#include "syslog.h"
+#include "utils.h"
 
 #include "internal.h"
-#include "nbd-protocol.h"
 #include "options.h"
-#include "exit-with-parent.h"
 
 #ifdef ENABLE_LIBFUZZER
 #define main fuzzer_main
@@ -78,7 +86,9 @@ static void write_pidfile (void);
 static bool is_config_key (const char *key, size_t len);
 static void error_if_stdio_closed (void);
 static void switch_stdio (void);
+static void winsock_init (void);
 
+int tcpip_sock_af = AF_UNSPEC;  /* -4, -6 */
 struct debug_flag *debug_flags; /* -D */
 bool exit_with_parent;          /* --exit-with-parent */
 const char *export_name;        /* -e */
@@ -129,13 +139,35 @@ usage (void)
 static void
 display_version (void)
 {
-  printf ("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
+  if (strcmp (NBDKIT_VERSION_EXTRA, "") == 0)
+    printf ("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
+  else
+    printf ("%s %s (%s)\n",
+            PACKAGE_NAME, PACKAGE_VERSION, NBDKIT_VERSION_EXTRA);
 }
 
 static void
 dump_config (void)
 {
-  CLEANUP_FREE char *binary = realpath ("/proc/self/exe", NULL);
+  CLEANUP_FREE char *binary = NULL;
+
+#ifdef __linux__
+  binary = realpath ("/proc/self/exe", NULL);
+#else
+#ifdef WIN32
+  /* GetModuleFileNameA has a crappy interface that prevents us from
+   * getting the length of the path so we just have to guess at an
+   * upper limit here.  It will at least truncate it properly with \0.
+   * _get_pgmptr would be a better alternative except that it isn't
+   * implemented in MinGW.  XXX
+   */
+  binary = malloc (256);
+  if (!GetModuleFileNameA (NULL, binary, 256)) {
+    free (binary);
+    binary = NULL;
+  }
+#endif
+#endif
 
   if (binary != NULL)
     printf ("%s=%s\n", "binary", binary);
@@ -161,6 +193,8 @@ dump_config (void)
   printf ("tls=no\n");
 #endif
   printf ("%s=%s\n", "version", PACKAGE_VERSION);
+  if (strcmp (NBDKIT_VERSION_EXTRA, "") != 0)
+    printf ("%s=%s\n", "version_extra", NBDKIT_VERSION_EXTRA);
   printf ("%s=%d\n", "version_major", NBDKIT_VERSION_MAJOR);
   printf ("%s=%d\n", "version_minor", NBDKIT_VERSION_MINOR);
 #ifdef HAVE_LIBZSTD
@@ -187,6 +221,7 @@ main (int argc, char *argv[])
   const char *magic_config_key;
 
   error_if_stdio_closed ();
+  winsock_init ();
 
 #if !ENABLE_LIBFUZZER
   threadlocal_init ();
@@ -225,17 +260,18 @@ main (int argc, char *argv[])
       break;
 
     case EXIT_WITH_PARENT_OPTION:
-#ifdef HAVE_EXIT_WITH_PARENT
-      exit_with_parent = true;
-      foreground = true;
+      if (can_exit_with_parent ()) {
+        exit_with_parent = true;
+        foreground = true;
+      }
+      else {
+        fprintf (stderr,
+                 "%s: --exit-with-parent is not implemented "
+                 "for this operating system\n",
+                 program_name);
+        exit (EXIT_FAILURE);
+      }
       break;
-#else
-      fprintf (stderr,
-               "%s: --exit-with-parent is not implemented "
-               "for this operating system\n",
-               program_name);
-      exit (EXIT_FAILURE);
-#endif
 
     case FILTER_OPTION:
       {
@@ -322,11 +358,17 @@ main (int argc, char *argv[])
       break;
 
     case TLS_VERIFY_PEER_OPTION:
+#ifdef HAVE_GNUTLS_SESSION_SET_VERIFY_CERT
       tls_verify_peer = true;
+#else
+      nbdkit_error ("--tls-verify-peer: "
+                    "GnuTLS >= 3.4.6 is required for this feature");
+      exit (EXIT_FAILURE);
+#endif
       break;
 
     case VSOCK_OPTION:
-#ifdef AF_VSOCK
+#if defined(AF_VSOCK) && defined(VMADDR_CID_ANY)
       vsock = true;
       break;
 #else
@@ -335,6 +377,14 @@ main (int argc, char *argv[])
       exit (EXIT_FAILURE);
 #endif
 
+    case '4':
+      tcpip_sock_af = AF_INET;
+      break;
+
+    case '6':
+      tcpip_sock_af = AF_INET6;
+      break;
+
     case 'D':
       add_debug_flag (optarg);
       break;
@@ -404,6 +454,16 @@ main (int argc, char *argv[])
         exit (EXIT_FAILURE);
       }
       listen_stdin = true;
+#ifdef WIN32
+      /* This could be implemented with a bit of work.  The problem
+       * currently is that we try to use recv() on the stdio file
+       * descriptor which winsock does not support (nor Linux in
+       * fact).  We would need to implement a test to see if the file
+       * descriptor is a socket or not and use either read or recv as
+       * appropriate.
+       */
+      NOT_IMPLEMENTED_ON_WINDOWS ("-s");
+#endif
       break;
 
     case 't':
@@ -462,7 +522,8 @@ main (int argc, char *argv[])
       /* Incorrect use of --dump-plugin. */
       fprintf (stderr,
                "%s: use 'nbdkit plugin --dump-plugin' or\n"
-               "'nbdkit /path/to/plugin.so --dump-plugin'\n",
+               "'nbdkit /path/to/plugin." SOEXT " --dump-plugin' or\n"
+               "if you want to find out about the server use --dump-config\n",
                program_name);
       exit (EXIT_FAILURE);
     }
@@ -494,6 +555,12 @@ main (int argc, char *argv[])
   if (log_to == LOG_TO_SYSLOG || log_to == LOG_TO_DEFAULT)
     openlog (program_name, LOG_PID, 0);
 
+  /* Print the version in debug output, right after syslog initialization. */
+  if (strcmp (NBDKIT_VERSION_EXTRA, "") == 0)
+    debug ("%s %s", PACKAGE_NAME, PACKAGE_VERSION);
+  else
+    debug ("%s %s (%s)", PACKAGE_NAME, PACKAGE_VERSION, NBDKIT_VERSION_EXTRA);
+
   /* Initialize TLS. */
   crypto_init (tls_set_on_cli);
   assert (tls != -1);
@@ -501,14 +568,10 @@ main (int argc, char *argv[])
   /* Implement --exit-with-parent early in case plugin initialization
    * takes a long time and the parent exits during that time.
    */
-#ifdef HAVE_EXIT_WITH_PARENT
-  if (exit_with_parent) {
-    if (set_exit_with_parent () == -1) {
-      perror ("nbdkit: --exit-with-parent");
-      exit (EXIT_FAILURE);
-    }
+  if (set_exit_with_parent () == -1) {
+    perror ("nbdkit: --exit-with-parent");
+    exit (EXIT_FAILURE);
   }
-#endif
 
   /* If the user has mixed up -p/--run/-s/-U/--vsock options, then
    * give an error.
@@ -522,8 +585,7 @@ main (int argc, char *argv[])
       (listen_stdin && run) ||
       (listen_stdin && dump_plugin) ||
       (vsock && unixsocket) ||
-      (vsock && listen_stdin) ||
-      (vsock && run)) {
+      (vsock && listen_stdin)) {
     fprintf (stderr,
              "%s: --dump-plugin, -p, --run, -s, -U or --vsock options "
              "cannot be used in this combination\n",
@@ -631,15 +693,9 @@ main (int argc, char *argv[])
    *
    * Keys must live for the life of nbdkit.  Since we want to avoid
    * modifying argv (so that /proc/PID/cmdline remains sane) but we
-   * need to create a key from argv[i] = "key=value" we must save the
-   * keys in an array which is freed at the end of main().
+   * need to create a key from argv[i] = "key=value" we must intern
+   * the keys, which are then freed at the end of main().
    */
-  char **keys = calloc (argc, sizeof (char *));
-  if (keys == NULL) {
-    perror ("calloc");
-    exit (EXIT_FAILURE);
-  }
-
   magic_config_key = top->magic_config_key (top);
   for (i = 0; optind < argc; ++i, ++optind) {
     size_t n;
@@ -647,12 +703,10 @@ main (int argc, char *argv[])
     p = strchr (argv[optind], '=');
     n = p - argv[optind];
     if (p && is_config_key (argv[optind], n)) { /* Is it key=value? */
-      keys[optind] = strndup (argv[optind], n);
-      if (keys[optind] == NULL) {
-        perror ("strndup");
+      const char *key = nbdkit_strndup_intern (argv[optind], n);
+      if (key == NULL)
         exit (EXIT_FAILURE);
-      }
-      top->config (top, keys[optind], p+1);
+      top->config (top, key, p+1);
     }
     else if (magic_config_key == NULL) {
       if (i == 0)               /* magic script parameter */
@@ -677,9 +731,7 @@ main (int argc, char *argv[])
   if (dump_plugin) {
     top->dump_fields (top);
     top->free (top);
-    for (i = 1; i < argc; ++i)
-      free (keys[i]);
-    free (keys);
+    free_interns ();
     exit (EXIT_SUCCESS);
   }
 
@@ -698,6 +750,7 @@ main (int argc, char *argv[])
 
   start_serving ();
 
+  top->cleanup (top);
   top->free (top);
   top = NULL;
 
@@ -717,9 +770,7 @@ main (int argc, char *argv[])
   crypto_free ();
   close_quit_pipe ();
 
-  for (i = 1; i < argc; ++i)
-    free (keys[i]);
-  free (keys);
+  free_interns ();
 
   /* Note: Don't exit here, otherwise this won't work when compiled
    * for libFuzzer.
@@ -731,32 +782,35 @@ main (int argc, char *argv[])
 static char *
 make_random_fifo (void)
 {
-  char template[] = "/tmp/nbdkitXXXXXX";
   char *sock;
 
-  if (mkdtemp (template) == NULL) {
-    perror ("mkdtemp");
-    return NULL;
-  }
-
-  random_fifo_dir = strdup (template);
+  random_fifo_dir = make_temporary_directory ();
   if (random_fifo_dir == NULL) {
-    perror ("strdup");
+    perror ("make_temporary_directory");
     return NULL;
   }
 
-  if (asprintf (&random_fifo, "%s/socket", template) == -1) {
+  if (asprintf (&random_fifo, "%s" DIR_SEPARATOR_STR "socket",
+                random_fifo_dir) == -1) {
     perror ("asprintf");
-    return NULL;
+    goto error;
   }
 
   sock = strdup (random_fifo);
   if (sock == NULL) {
     perror ("strdup");
-    return NULL;
+    goto error;
   }
 
   return sock;
+
+ error:
+  free (random_fifo);
+  random_fifo = NULL;
+  rmdir (random_fifo_dir);
+  free (random_fifo_dir);
+  random_fifo_dir = NULL;
+  return NULL;
 }
 
 static struct backend *
@@ -772,7 +826,7 @@ open_plugin_so (size_t i, const char *name, int short_name)
   if (short_name) {
     /* Short names are rewritten relative to the plugindir. */
     if (asprintf (&filename,
-                  "%s/nbdkit-%s-plugin.so", plugindir, name) == -1) {
+                  "%s/nbdkit-%s-plugin." SOEXT, plugindir, name) == -1) {
       perror ("asprintf");
       exit (EXIT_FAILURE);
     }
@@ -792,7 +846,7 @@ open_plugin_so (size_t i, const char *name, int short_name)
 
   /* Initialize the plugin.  See dlopen(3) to understand C weirdness. */
   dlerror ();
-  *(void **) (&plugin_init) = dlsym (dl, "plugin_init");
+  plugin_init = dlsym (dl, "plugin_init");
   if ((error = dlerror ()) != NULL) {
     fprintf (stderr, "%s: %s: %s\n", program_name, name, error);
     exit (EXIT_FAILURE);
@@ -825,7 +879,7 @@ open_filter_so (struct backend *next, size_t i,
   if (short_name) {
     /* Short names are rewritten relative to the filterdir. */
     if (asprintf (&filename,
-                  "%s/nbdkit-%s-filter.so", filterdir, name) == -1) {
+                  "%s/nbdkit-%s-filter." SOEXT, filterdir, name) == -1) {
       perror ("asprintf");
       exit (EXIT_FAILURE);
     }
@@ -841,7 +895,7 @@ open_filter_so (struct backend *next, size_t i,
 
   /* Initialize the filter.  See dlopen(3) to understand C weirdness. */
   dlerror ();
-  *(void **) (&filter_init) = dlsym (dl, "filter_init");
+  filter_init = dlsym (dl, "filter_init");
   if ((error = dlerror ()) != NULL) {
     fprintf (stderr, "%s: %s: %s\n", program_name, name, error);
     exit (EXIT_FAILURE);
@@ -901,7 +955,7 @@ start_serving (void)
       r = sockets_append (&socks, s);
       assert (r == 0);
     }
-    debug ("using socket activation, nr_socks = %zu", socks.size);
+    debug ("using socket activation, nr_socks = %zu", socks.len);
     change_user ();
     write_pidfile ();
     top->after_fork (top);
@@ -949,7 +1003,16 @@ write_pidfile (void)
     return;
 
   pid = getpid ();
-  snprintf (pidstr, sizeof pidstr, "%d\n", pid);
+  /* Don't put a trailing \n after the PID file on Windows. It is
+   * turned into \r\n which causes problems if you process the file
+   * using a Unix tool like bash, especially when running the test
+   * suite.
+   */
+  snprintf (pidstr, sizeof pidstr, "%d"
+#ifndef WIN32
+            "\n"
+#endif
+            , (int) pid);
   len = strlen (pidstr);
 
   fd = open (pidfile, O_WRONLY|O_TRUNC|O_CREAT|O_CLOEXEC|O_NOCTTY, 0644);
@@ -1000,6 +1063,7 @@ is_config_key (const char *key, size_t len)
 static void
 error_if_stdio_closed (void)
 {
+#ifdef F_GETFL
   if (fcntl (STDERR_FILENO, F_GETFL) == -1) {
     /* Nowhere we can report the error. Oh well. */
     exit (EXIT_FAILURE);
@@ -1009,6 +1073,7 @@ error_if_stdio_closed (void)
     perror ("expecting stdin/stdout to be opened");
     exit (EXIT_FAILURE);
   }
+#endif
 }
 
 /* Sanitize stdin/stdout to /dev/null, after saving the originals
@@ -1023,6 +1088,7 @@ error_if_stdio_closed (void)
 static void
 switch_stdio (void)
 {
+#if defined(F_DUPFD_CLOEXEC) || defined(F_DUPFD)
   fflush (stdin);
   fflush (NULL);
   if (listen_stdin || run) {
@@ -1040,6 +1106,8 @@ switch_stdio (void)
       exit (EXIT_FAILURE);
     }
   }
+#endif
+#ifndef WIN32
   close (STDIN_FILENO);
   close (STDOUT_FILENO);
   if (open ("/dev/null", O_RDONLY) != STDIN_FILENO ||
@@ -1047,4 +1115,23 @@ switch_stdio (void)
     perror ("open");
     exit (EXIT_FAILURE);
   }
+#endif
+}
+
+/* On Windows the Winsock library must be initialized early.
+ * https://docs.microsoft.com/en-us/windows/win32/winsock/initializing-winsock
+ */
+static void
+winsock_init (void)
+{
+#ifdef WIN32
+  WSADATA wsaData;
+  int result;
+
+  result = WSAStartup (MAKEWORD (2, 2), &wsaData);
+  if (result != 0) {
+    fprintf (stderr, "WSAStartup failed: %d\n", result);
+    exit (EXIT_FAILURE);
+  }
+#endif
 }
