@@ -53,11 +53,14 @@ requires $OCAMLOPT -version
 requires_nbdsh_uri
 requires nbdinfo --version
 
+# For unclear reasons linking the OCaml plugin fails on macOS. XXX
+requires_not test "$(uname)" = "Darwin"
+
 out=test-cc-ocaml.out
 cleanup_fn rm -f $out
 rm -f $out
 
-nbdkit -U - cc $script a=1 b=2 c=3 \
-       CC="$OCAMLOPT" CFLAGS="-output-obj -runtime-variant _pic -I $SRCDIR/../plugins/ocaml NBDKit.cmx -cclib -lnbdkitocaml" \
+nbdkit -U - cc $script a=1 b=2 c=3 d=4 \
+       CC="$OCAMLOPT" CFLAGS="-output-obj -runtime-variant _pic -I $SRCDIR/../plugins/ocaml unix.cmxa NBDKit.cmx -cclib -lnbdkitocaml" \
        --run 'nbdinfo --size $uri' > $out
 test "$(cat $out)" -eq $((512 * 2048))
